@@ -1,20 +1,10 @@
+from typing import Dict, List, Optional, Tuple, Union
 
-
-from typing import List
-from spacy.tokens import Doc, Span
-
-
-
-
-
-
-from typing import Dict, Optional, Tuple, List, Union
-from spacy.tokens import Doc, Span, Token
-from spacy.matcher import PhraseMatcher
-from spacy.language import Language
-from spacy.util import filter_spans
 from emoji import UNICODE_EMOJI
-
+from spacy.language import Language
+from spacy.matcher import PhraseMatcher
+from spacy.tokens import Doc, Span, Token
+from spacy.util import filter_spans
 
 # Make sure multi-character emoji don't contain whitespace
 EMOJI = {e.replace(" ", ""): t for e, t in UNICODE_EMOJI.items()}
@@ -38,7 +28,8 @@ def create_twitter(
 
 
 class Twitter:
-    """spaCy pipeline component for added hashtags, mentions and urls to `Doc`objects.
+    """spaCy pipeline component for added hashtags, mentions and urls to
+    `Doc`objects.
 
     Examples:
         >>> import spacy
@@ -69,24 +60,25 @@ class Twitter:
     def __init__(
         self,
         nlp: Language,
-        redo_tokenization : bool = True,
+        redo_tokenization: bool = True,
         lookup: Optional[Dict[str, str]] = None,
         pattern_id: str = "EMOJI",
         attrs: Tuple[str, str, str, str] = DEFAULT_ATTRS,
         force_extension: bool = True,
     ) -> None:
         """Initialise the pipeline component.
-        nlp (Language): The shared nlp object. Used to initialise the matcher
-            with the shared `Vocab`, and create `Doc` match patterns.
-        attrs (tuple): Attributes to set on the ._ property. Defaults to
-            ('has_emoji', 'is_emoji', 'emoji_desc', 'emoji').
-        pattern_id (unicode): ID of match pattern, defaults to 'EMOJI'. Can be
-            changed to avoid ID clashes.
-        merge_spans (bool): Merge spans containing multi-character emoji. Will
-            only merge combined emoji resulting in one icon, not sequences.
-        lookup (dict): Optional lookup table that maps emoji unicode strings
-            to custom descriptions, e.g. translations or other annotations.
-        RETURNS (callable): A spaCy pipeline component.
+
+        nlp (Language): The shared nlp object. Used to initialise the
+        matcher     with the shared `Vocab`, and create `Doc` match
+        patterns. attrs (tuple): Attributes to set on the ._ property.
+        Defaults to     ('has_emoji', 'is_emoji', 'emoji_desc',
+        'emoji'). pattern_id (unicode): ID of match pattern, defaults to
+        'EMOJI'. Can be     changed to avoid ID clashes. merge_spans
+        (bool): Merge spans containing multi-character emoji. Will
+        only merge combined emoji resulting in one icon, not sequences.
+        lookup (dict): Optional lookup table that maps emoji unicode
+        strings     to custom descriptions, e.g. translations or other
+        annotations. RETURNS (callable): A spaCy pipeline component.
         """
         self._has_emoji, self._is_emoji, self._emoji_desc, self._emoji = attrs
         self.merge_spans = merge_spans
@@ -98,18 +90,23 @@ class Twitter:
         Doc.set_extension(self._has_emoji, getter=self.has_emoji, force=force_extension)
         Doc.set_extension(self._emoji, getter=self.iter_emoji, force=force_extension)
         Span.set_extension(
-            self._has_emoji, getter=self.has_emoji, force=force_extension
+            self._has_emoji,
+            getter=self.has_emoji,
+            force=force_extension,
         )
         Span.set_extension(self._emoji, getter=self.iter_emoji, force=force_extension)
         Token.set_extension(self._is_emoji, default=False, force=force_extension)
         Token.set_extension(
-            self._emoji_desc, getter=self.get_emoji_desc, force=force_extension
+            self._emoji_desc,
+            getter=self.get_emoji_desc,
+            force=force_extension,
         )
 
     def __call__(self, doc: Doc) -> Doc:
         """Apply the pipeline component to a `Doc` object.
-        doc (Doc): The `Doc` returned by the previous pipeline component.
-        RETURNS (Doc): The modified `Doc` object.
+
+        doc (Doc): The `Doc` returned by the previous pipeline
+        component. RETURNS (Doc): The modified `Doc` object.
         """
         spans = self.matcher(doc, as_spans=True)
         for span in spans:
@@ -144,10 +141,8 @@ class Twitter:
         return None
 
 
-
 def hashtag_getter(doc: Doc) -> List[str]:
-    """
-    Extract hashtags from text
+    """Extract hashtags from text.
 
     Args:
         doc (Doc): A SpaCy document
@@ -162,18 +157,32 @@ def hashtag_getter(doc: Doc) -> List[str]:
         >>> doc._.hashtag  # extrac the hashtags from your document
         ["#yolo", "#life"]
     """
-    def find_hashtags(text, valid_tags = {"#", "＃"}, valid_chars = {"_", "-"}, invalid_tag_suffix = {b'\xe2\x83\xa3', b'\xef\xb8\x8f'}):
+
+    def find_hashtags(
+        text,
+        valid_tags={"#", "＃"},
+        valid_chars={"_", "-"},
+        invalid_tag_suffix={b"\xe2\x83\xa3", b"\xef\xb8\x8f"},
+    ):
         def is_letter(t):
-            if t.isalnum() or t in valid_chars or str.encode(t).startswith(b"\xcc") or str.encode(t).startswith(b"\xe0"):
+            if (
+                t.isalnum()
+                or t in valid_chars
+                or str.encode(t).startswith(b"\xcc")
+                or str.encode(t).startswith(b"\xe0")
+            ):
                 return True
             return False
 
-        
         start = None
         for i, char in enumerate(text):
-            if (char in valid_tags and 
-                not (i+1 != len(text) and str.encode(text[i+1]) in invalid_tag_suffix) and 
-                (i == 0 or not (is_letter(text[i-1]) or text[i-1] == "&"))):
+            if (
+                char in valid_tags
+                and not (
+                    i + 1 != len(text) and str.encode(text[i + 1]) in invalid_tag_suffix
+                )
+                and (i == 0 or not (is_letter(text[i - 1]) or text[i - 1] == "&"))
+            ):
                 start = i
                 continue
             if start is not None and not is_letter(char):
@@ -181,17 +190,16 @@ def hashtag_getter(doc: Doc) -> List[str]:
                     start = None
                     continue
                 print(start, i)
-                if not text[start+1: i].isnumeric():
-                    yield "#" + text[start+1: i]
+                if not text[start + 1 : i].isnumeric():
+                    yield "#" + text[start + 1 : i]
                 start = None
-        if start is not None and not text[start+1: i+1].isnumeric():
+        if start is not None and not text[start + 1 : i + 1].isnumeric():
             print(start, i)
-            yield "#" + text[start+1: i+1]
+            yield "#" + text[start + 1 : i + 1]
 
     return list(find_hashtags(doc.text))
 
 
-
 # def url_getter(doc: Doc) -> List[str]:
 #     """
 #     Extract hashtags from text
@@ -246,18 +254,31 @@ def hashtag_getter(doc: Doc) -> List[str]:
 #     return LIX
 
 
-def find_hashtags(text, valid_tags = {"#", "＃"}, valid_chars = {"_", "-"}, invalid_tag_suffix = {b'\xe2\x83\xa3', b'\xef\xb8\x8f'}):
+def find_hashtags(
+    text,
+    valid_tags={"#", "＃"},
+    valid_chars={"_", "-"},
+    invalid_tag_suffix={b"\xe2\x83\xa3", b"\xef\xb8\x8f"},
+):
     def is_letter(t):
-        if t.isalnum() or t in valid_chars or str.encode(t).startswith(b"\xcc") or str.encode(t).startswith(b"\xe0"):
+        if (
+            t.isalnum()
+            or t in valid_chars
+            or str.encode(t).startswith(b"\xcc")
+            or str.encode(t).startswith(b"\xe0")
+        ):
             return True
         return False
 
-    
     start = None
     for i, char in enumerate(text):
-        if (char in valid_tags and 
-            not (i+1 != len(text) and str.encode(text[i+1]) in invalid_tag_suffix) and 
-            (i == 0 or not (is_letter(text[i-1]) or text[i-1] == "&"))):
+        if (
+            char in valid_tags
+            and not (
+                i + 1 != len(text) and str.encode(text[i + 1]) in invalid_tag_suffix
+            )
+            and (i == 0 or not (is_letter(text[i - 1]) or text[i - 1] == "&"))
+        ):
             start = i
             continue
         if start is not None and not is_letter(char):
@@ -265,20 +286,23 @@ def find_hashtags(text, valid_tags = {"#", "＃"}, valid_chars = {"_", "-"}, inv
                 start = None
                 continue
             print(start, i)
-            if not text[start+1: i].isnumeric():
-                yield "#" + text[start+1: i]
+            if not text[start + 1 : i].isnumeric():
+                yield "#" + text[start + 1 : i]
             start = None
-    if start is not None and not text[start+1: i+1].isnumeric():
+    if start is not None and not text[start + 1 : i + 1].isnumeric():
         print(start, i)
-        yield "#" + text[start+1: i+1]
+        yield "#" + text[start + 1 : i + 1]
 
-for s in ["Fuck hvor fedt! #yolo #life", 
-          "#taga",
-          "invalid hashtags #double#tag",
-          "invalid hashtags #double@tag",
-          "also invalid #1"]:
+
+for s in [
+    "Fuck hvor fedt! #yolo #life",
+    "#taga",
+    "invalid hashtags #double#tag",
+    "invalid hashtags #double@tag",
+    "also invalid #1",
+]:
     result = list(find_hashtags(s))
-    print(s, "\t", result) 
+    print(s, "\t", result)
 
 # from spacy.lang.da import Danish
 # nlp = Danish()
@@ -573,16 +597,21 @@ hash_tag_examples = """"
 """
 
 import re
+
 lines = hash_tag_examples.split("\n")
 examples = []
 for i, line in enumerate(lines):
     if line.startswith("    - description"):
         rex = re.compile(r'<a.*? title="(.*?)" .*?<\/a>')
-        match = rex.findall(lines[i+2][17:-1])
-        examples.append({"desc": lines[i],
-                         "text": lines[i+1][13:-1], 
-                         "expected": lines[i+2][17:-1],
-                         "hashtags": match})
+        match = rex.findall(lines[i + 2][17:-1])
+        examples.append(
+            {
+                "desc": lines[i],
+                "text": lines[i + 1][13:-1],
+                "expected": lines[i + 2][17:-1],
+                "hashtags": match,
+            },
+        )
 
 
 for i, d in enumerate(examples):
@@ -615,75 +644,81 @@ str.encode("áim"[1])
 "â"[0]
 
 
-hashtag_tests = [('text #hashtag', ['#hashtag']),
- ('text #hash0tag', ['#hash0tag']),
- ('text #1tag', ['#1tag']),
- ('text #hash_tag', ['#hash_tag']),
- ('text #1234', []),
- ('text#hashtag', []),
- ('#️hashtag', []),
- ('#⃣hashtag', []),
- ('text #hashtag1 #hashtag2', ['#hashtag1', '#hashtag2']),
- ('text.#hashtag', ['#hashtag']),
- ('&#nbsp;', []),
- ('text #hashtag!', ['#hashtag']),
- ('text #dodge/#answer', ['#dodge', '#answer']),
- ('text #dodge/answer', ['#dodge']),
- ('text dodge/#answer', ['#answer']),
- ('text #hashtagの', ['#hashtagの']),
- ('text\u3000#hashtag', ['#hashtag']),
- ('#hashtag\u3000text', ['#hashtag']),
- ('＃hashtag', ['#hashtag']),
- ('#éhashtag', ['#éhashtag']),
- ('#hashtagé', ['#hashtagé']),
- ('#hashétag', ['#hashétag']),
- ('What is #트위터 anyway?', ['#트위터']),
- ('What is #ашок anyway?', ['#ашок']),
- ('カタカナ #カタカナ カタカナ', ['#カタカナ']),
- ('カタカナ #カタカナ」カタカナ', ['#カタカナ']),
- ('カタカナ #カタカナ', ['#カタカナ']),
- ('カタカナ「#カタカナ カタカナ', ['#カタカナ']),
- ('カタカナ「#カタカナ」カタカナ', ['#カタカナ']),
- ('カタカナ「#カタカナ', ['#カタカナ']),
- ('#カタカナ カタカナ', ['#カタカナ']),
- ('#カタカナ」カタカナ', ['#カタカナ']),
- ('#カタカナ', ['#カタカナ']),
- ('#ﾊｯｼｭﾀｸﾞ\u3000テスト', ['#ﾊｯｼｭﾀｸﾞ']),
- ('#ﾊｯｼｭﾀｸﾞ123', ['#ﾊｯｼｭﾀｸﾞ123']),
- ('#ﾊﾟﾋﾟﾌﾟﾍﾟﾎﾟ', ['#ﾊﾟﾋﾟﾌﾟﾍﾟﾎﾟ']),
- ('漢字 #漢字 漢字', ['#漢字']),
- ('漢字 #漢字」漢字', ['#漢字']),
- ('漢字 #漢字', ['#漢字']),
- ('漢字「#漢字 漢字', ['#漢字']),
- ('漢字「#漢字」漢字', ['#漢字']),
- ('漢字「#漢字', ['#漢字']),
- ('#漢字 漢字', ['#漢字']),
- ('#漢字」漢字', ['#漢字']),
- ('#漢字', ['#漢字']),
- ('これは、＃大丈夫。', ['#大丈夫']),
- ('ひらがな #ひらがな ひらがな', ['#ひらがな']),
- ('ひらがな #ひらがな」ひらがな', ['#ひらがな']),
- ('ひらがな #ひらがな', ['#ひらがな']),
- ('ひらがな「#ひらがな ひらがな', ['#ひらがな']),
- ('ひらがな「#ひらがな」ひらがな', ['#ひらがな']),
- ('ひらがな「#ひらがな', ['#ひらがな']),
- ('#ひらがな ひらがな', ['#ひらがな']),
- ('#ひらがな」ひらがな', ['#ひらがな']),
- ('#ひらがな', ['#ひらがな']),
- ('日本語ハッシュタグ #日本語ハッシュタグ', ['#日本語ハッシュタグ']),
- ('日本語ハッシュタグ#日本語ハッシュタグ', []),
- ('#日本語ハッシュタグ。', ['#日本語ハッシュタグ']),
- ('日本語ハッシュタグ。#日本語ハッシュタグ', ['#日本語ハッシュタグ']),
- ('長音ハッシュタグ。#サッカー', ['#サッカー']),
- ('長音ハッシュタグ。#ｻｯｶｰ', ['#ｻｯｶｰ']),
- ('できましたよー！#日本語ハッシュタグ。', ['#日本語ハッシュタグ']),
- ('できましたよー！＃日本語ハッシュタグ。', ['#日本語ハッシュタグ']),
- ('#云々', ['#云々']),
- ('Hashtags in #中文, #日本語, #한국말, and #русский! Try it out!',
-  ['#中文', '#日本語', '#한국말', '#русский']),
- ('Here’s a test tweet for you: #Ateş #qrşt #ştu #ş',
-  ['#Ateş', '#qrşt', '#ştu', '#ş']),
- ('#mûǁae', ['#mûǁae']),
- ('#táim #hag̃ua', ['#táim', '#hag̃ua']),
- ('Arabic hashtag: #فارسی #لس_آنجلس', ['#فارسی', '#لس_آنجلس']),
- ('Thai hashtag: #รายละเอียด', ['#รายละเอียด'])]
\ No newline at end of file
+hashtag_tests = [
+    ("text #hashtag", ["#hashtag"]),
+    ("text #hash0tag", ["#hash0tag"]),
+    ("text #1tag", ["#1tag"]),
+    ("text #hash_tag", ["#hash_tag"]),
+    ("text #1234", []),
+    ("text#hashtag", []),
+    ("#️hashtag", []),
+    ("#⃣hashtag", []),
+    ("text #hashtag1 #hashtag2", ["#hashtag1", "#hashtag2"]),
+    ("text.#hashtag", ["#hashtag"]),
+    ("&#nbsp;", []),
+    ("text #hashtag!", ["#hashtag"]),
+    ("text #dodge/#answer", ["#dodge", "#answer"]),
+    ("text #dodge/answer", ["#dodge"]),
+    ("text dodge/#answer", ["#answer"]),
+    ("text #hashtagの", ["#hashtagの"]),
+    ("text\u3000#hashtag", ["#hashtag"]),
+    ("#hashtag\u3000text", ["#hashtag"]),
+    ("＃hashtag", ["#hashtag"]),
+    ("#éhashtag", ["#éhashtag"]),
+    ("#hashtagé", ["#hashtagé"]),
+    ("#hashétag", ["#hashétag"]),
+    ("What is #트위터 anyway?", ["#트위터"]),
+    ("What is #ашок anyway?", ["#ашок"]),
+    ("カタカナ #カタカナ カタカナ", ["#カタカナ"]),
+    ("カタカナ #カタカナ」カタカナ", ["#カタカナ"]),
+    ("カタカナ #カタカナ", ["#カタカナ"]),
+    ("カタカナ「#カタカナ カタカナ", ["#カタカナ"]),
+    ("カタカナ「#カタカナ」カタカナ", ["#カタカナ"]),
+    ("カタカナ「#カタカナ", ["#カタカナ"]),
+    ("#カタカナ カタカナ", ["#カタカナ"]),
+    ("#カタカナ」カタカナ", ["#カタカナ"]),
+    ("#カタカナ", ["#カタカナ"]),
+    ("#ﾊｯｼｭﾀｸﾞ\u3000テスト", ["#ﾊｯｼｭﾀｸﾞ"]),
+    ("#ﾊｯｼｭﾀｸﾞ123", ["#ﾊｯｼｭﾀｸﾞ123"]),
+    ("#ﾊﾟﾋﾟﾌﾟﾍﾟﾎﾟ", ["#ﾊﾟﾋﾟﾌﾟﾍﾟﾎﾟ"]),
+    ("漢字 #漢字 漢字", ["#漢字"]),
+    ("漢字 #漢字」漢字", ["#漢字"]),
+    ("漢字 #漢字", ["#漢字"]),
+    ("漢字「#漢字 漢字", ["#漢字"]),
+    ("漢字「#漢字」漢字", ["#漢字"]),
+    ("漢字「#漢字", ["#漢字"]),
+    ("#漢字 漢字", ["#漢字"]),
+    ("#漢字」漢字", ["#漢字"]),
+    ("#漢字", ["#漢字"]),
+    ("これは、＃大丈夫。", ["#大丈夫"]),
+    ("ひらがな #ひらがな ひらがな", ["#ひらがな"]),
+    ("ひらがな #ひらがな」ひらがな", ["#ひらがな"]),
+    ("ひらがな #ひらがな", ["#ひらがな"]),
+    ("ひらがな「#ひらがな ひらがな", ["#ひらがな"]),
+    ("ひらがな「#ひらがな」ひらがな", ["#ひらがな"]),
+    ("ひらがな「#ひらがな", ["#ひらがな"]),
+    ("#ひらがな ひらがな", ["#ひらがな"]),
+    ("#ひらがな」ひらがな", ["#ひらがな"]),
+    ("#ひらがな", ["#ひらがな"]),
+    ("日本語ハッシュタグ #日本語ハッシュタグ", ["#日本語ハッシュタグ"]),
+    ("日本語ハッシュタグ#日本語ハッシュタグ", []),
+    ("#日本語ハッシュタグ。", ["#日本語ハッシュタグ"]),
+    ("日本語ハッシュタグ。#日本語ハッシュタグ", ["#日本語ハッシュタグ"]),
+    ("長音ハッシュタグ。#サッカー", ["#サッカー"]),
+    ("長音ハッシュタグ。#ｻｯｶｰ", ["#ｻｯｶｰ"]),
+    ("できましたよー！#日本語ハッシュタグ。", ["#日本語ハッシュタグ"]),
+    ("できましたよー！＃日本語ハッシュタグ。", ["#日本語ハッシュタグ"]),
+    ("#云々", ["#云々"]),
+    (
+        "Hashtags in #中文, #日本語, #한국말, and #русский! Try it out!",
+        ["#中文", "#日本語", "#한국말", "#русский"],
+    ),
+    (
+        "Here’s a test tweet for you: #Ateş #qrşt #ştu #ş",
+        ["#Ateş", "#qrşt", "#ştu", "#ş"],
+    ),
+    ("#mûǁae", ["#mûǁae"]),
+    ("#táim #hag̃ua", ["#táim", "#hag̃ua"]),
+    ("Arabic hashtag: #فارسی #لس_آنجلس", ["#فارسی", "#لس_آنجلس"]),
+    ("Thai hashtag: #รายละเอียด", ["#รายละเอียด"]),
+]
